// $Id: Playfield.cc,v 1.36 1999/12/13 22:54:41 grumbel Exp $
//
// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <iostream>
#include <cstdio>
#include <cassert>

#include "globals.hh"
#include "actions/pingu_actions.hh"
#include "Pingu.hh"
#include "Playfield.hh"
#include "PLF.hh"
#include "Result.hh"
#include "LevelInterrupt.hh"
#include "CaptureRectangle.hh"
#include "GameTime.hh"
#include "QuestionDialog.hh"

Playfield::Playfield()
{
  current_pingu = 0;
  x_offset = 0;
  y_offset = 0;
  server = 0;
  client = 0;
  //  pause = false;
}

Playfield::Playfield(PLF* level_data, World* w)
{
  world = w;
  current_pingu = 0;
  current_view = 0;
  pingus = world->get_pingu_p();

  x_offset = 0; 
  y_offset = 0; 

  x_offset2 = x_offset;
  y_offset2 = y_offset;
  //  wav = CL_Streamed_MikModSample::create("../data/music/gd-walk.it",
					 //					 NULL, true);
  //  ses = new CL_SoundBuffer_Session( wav->prepare() );
  //  ses->play();
  View::set_world(world);

  // Create a default view
  {
    int x1, x2, y1, y2;

    x1 = (CL_Display::get_width() - world->get_width()) / 2;
    x2 = x1 + world->get_width();

    y1 = (CL_Display::get_height() - world->get_height()) / 2;
    y2 = y1 + world->get_height();

    if (x1 < 0)  
      x1 = 0;
    if (x2 > CL_Display::get_width() - 1) 
      x2 =  CL_Display::get_width() - 1;
    if (y1 < 0)
      y1 = 0;
    if (y2 >  CL_Display::get_height())
      y2 =  CL_Display::get_height();

    if (verbose) {
      cout << "Playfield: Using screen size: " 
	   << x1 << ", "
	   << y1 << ", " 
	   << x2 << ", "
	   << y2 << endl;
    }
    
    if (gimmicks_enabled) {
      if (verbose)
	cout << "Playfield: Using gimmick" << endl;
      view.push_back(View(x2/2, y1, x2, y2));
      view.push_back(View(0, 21, x2/2, y2/2, 0.5));
      view.push_back(View(0, y2/2, x2/2, y2, 2.0));
    } else { // !gimmicks_enabled
      view.push_back(View(x1, y1, x2, y2));

      cout << "Start_X: " << ((x2 - x1) / 2) - level_data->get_startx() << endl;
      cout << "Start_Y: " << ((y2 - y1) / 2) - level_data->get_starty() << endl;

      view[0].set_x_offset(((x2 - x1) / 2) - level_data->get_startx());
      view[0].set_y_offset(((y2 - y1) / 2) - level_data->get_starty());
    }
  }
}

Playfield::~Playfield()
{
  //  delete cap;

  if (verbose)
    cout << "Playfield going down" << endl;
  //  write_results();
}

void
Playfield::draw()
{ 
  for(unsigned int i=0; i < view.size(); ++i) {
    view[i].draw();
  }
}

// x1 and y1 are the offsets used to calculate the world co's
Pingu*
Playfield::current_pingu_find(int x1, int y1, double s)
{
  for (PinguIter pingu=pingus->begin(); pingu != pingus->end(); ++pingu) {
    if ((*pingu)->is_over(CL_Mouse::get_x() / s + x1,
			  CL_Mouse::get_y() / s + y1)
	&& (*pingu)->get_status() != dead)
      {
	return *pingu;
      }
  }
  return 0; 
}

void
Playfield::set_world(World* w)
{
  world = w;
}

void 
Playfield::process_input()
{
  process_input_interactive();
}

void
Playfield::process_input_demomode()
{
}

void 
Playfield::process_input_interactive()
{
  static int scroll_center_x;
  static int scroll_center_y;

  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_PAGEDOWN)) {
    view[current_view].set_zoom(view[current_view].get_zoom() / 1.05);
  } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_PAGEUP)) {
    view[current_view].set_zoom(view[current_view].get_zoom() / 0.95);
  } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_KP_ENTER)) {
    view[current_view].set_zoom(1.0);
  }
  /*
  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F1)) {
    CL_Font* font = CL_Res_Font::load("Fonts/smallfont_h",*resource());
    CL_Display::sync_buffers();
    CL_Display::fill_rect(0,0, CL_Display::get_width(), CL_Display::get_height(),
			  0.0, 0.0, 0.0, 0.5);
    
    while(CL_Keyboard::get_keycode(CL_Keyboard::KEY_F1))
      CL_System::keep_alive();

    font->print_left(100, 40, "Key    Function");
    font->print_left(100, 70, " a     Armageddon - let all pingus explode ");
    font->print_left(100, 90, " r     restart level");
    font->print_left(100,110, " l     scroll around in the level");
    font->print_left(100,130, " n     load a new level");
    font->print_left(100,150, "ESC    exit/restart/load level");
    font->print_left(100,170, "F1     display this help");

    CL_Display::flip_display();

    while(!CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
      CL_System::keep_alive();

    while(CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
      CL_System::keep_alive();
  }
  */
  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_LSHIFT) 
      || CL_Keyboard::get_keycode(CL_Keyboard::KEY_RSHIFT)) 
    {
      scroll_speed = 50;
    }
  else 
    {
      scroll_speed = 15;
    }


  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_LEFT)) {
    view[current_view].set_x_offset(view[current_view].get_x_offset() + scroll_speed);
  } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_RIGHT)) {
    view[current_view].set_x_offset(view[current_view].get_x_offset() - scroll_speed);
  }

  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_UP)) {
    view[current_view].set_y_offset(view[current_view].get_y_offset() + scroll_speed);
  } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_DOWN)) {
    view[current_view].set_y_offset(view[current_view].get_y_offset() - scroll_speed);
  }

  // Scroll around if the mouse button is pressed
  if (CL_Mouse::right_pressed() || CL_Keyboard::get_keycode(CL_Keyboard::KEY_LSHIFT)) {
    if (!mouse_scrolling) {
      char str[1024];

      mouse_scrolling = true;
      scroll_center_x = CL_Mouse::get_x();
      scroll_center_y = CL_Mouse::get_y();
      x_offset = view[current_view].get_x_offset();
      y_offset = view[current_view].get_y_offset();

      sprintf(str, "Playfield: Start scrolling: X: %d Y: %d", x_offset, y_offset);
      //      event.send(str);
    }
    x_offset += (scroll_center_x - CL_Mouse::get_x()) / 5;
    y_offset += (scroll_center_y - CL_Mouse::get_y()) / 5;
    
    view[current_view].set_x_offset(x_offset);
    view[current_view].set_y_offset(y_offset);
  } else {
    if (mouse_scrolling) {
      char str[1024];
      sprintf(str, "Playfield: Stop scrolling: X: %d Y: %d", x_offset, y_offset);
      //      event.send(str);
    }
    mouse_scrolling = false;
  }

  // BUG: This should be replaced with something getting relative mouse co's
  if (CL_Mouse::get_x() < 2) {
    view[current_view].set_x_offset(view[current_view].get_x_offset() + scroll_speed);
  } else if (CL_Mouse::get_x() > CL_Display::get_width() - 3) {
    view[current_view].set_x_offset(view[current_view].get_x_offset() - scroll_speed);
  }

  if (CL_Mouse::get_y() < 2) {
    view[current_view].set_y_offset(view[current_view].get_y_offset() + scroll_speed);
  } else if (CL_Mouse::get_y() > CL_Display::get_height() - 3) {
    view[current_view].set_y_offset(view[current_view].get_y_offset() - scroll_speed);
  }
  // BUGEND

  if (CL_Mouse::middle_pressed()) {
    cout << "Mouse Coordinates ="
	 << " X: " << CL_Mouse::get_x() - x_offset
	 << " Y: " << CL_Mouse::get_y() - y_offset << endl;
    world->print_status();
  }
  
  // Let the playfield scroll around
  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_LEFT)) {
    x_offset2 += 10;
  } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_RIGHT)) {
    x_offset2 -= 10;
  } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_UP)) {
    y_offset2 += 10;
  } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_DOWN)) {
    y_offset2 -= 10;
  }
  
  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_R)) {
    cout << "Going to start again" << endl;
    // write_results();
    //throw LevelInterrupt(LevelInterrupt::restart);
    client->do_restart();
  }

  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE)) {
    while (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE)) {
      CL_System::keep_alive();
    }
    cout << "Going done" << endl;
    server->set_finished();
    /*
    CL_Display::sync_buffers();
    CL_Display::flip_display();
    QuestionDialog dialog("exit level?", "yes", "no", "restart", "load");
    while (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE)) {
      CL_System::keep_alive();
    }
    switch(dialog.start()) {
    case 0: // yes
      //throw LevelInterrupt(LevelInterrupt::quit);
      break;
    case 1: // no
      break;
    case 2: // restart
      //throw LevelInterrupt(LevelInterrupt::restart);
      break;
    case 3: // select
      //throw LevelInterrupt(LevelInterrupt::select_new);
      break;
    }     
    */
  }

  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_A)) {
    assert(server);
    server->send_event("armageddon");
    //    event.send("aramageddon");
    //    world->armageddon();
  }  
}

/*bool
Playfield::enough_time_passed()
{
  static int last_time = 0;
  
  if (last_time + game_speed > CL_System::get_time())
    return false;
  else {
    last_time = CL_System::get_time();
    return true;
  }
}
*/
void
Playfield::let_move()
{
  process_input();

  // FIXME: This could need some optimization, copying strings all
  // around is bad and also we have hear a memory hole.
  CaptureRectangle::set_action(makePinguAction(buttons->get_action_name())); 

  for(unsigned int i=0; i < view.size(); ++i) {
    if (view[i].is_current() && !mouse_scrolling) { 
      current_view = i;
      current_pingu = current_pingu_find(-view[i].get_x_pos() - (view[i].get_x_offset()),
					 -view[i].get_y_pos() - (view[i].get_y_offset()), view[i].get_zoom()); 
    
      //pingu_info->set_pingu(current_pingu);
      view[i].set_pingu(current_pingu);
      break;
    }
  }
  
  if (current_pingu && CL_Mouse::left_pressed()) {
    char str[256];
    sprintf(str, "Pingu:%d:%s", current_pingu->get_id(), buttons->get_action_name().c_str());
    server->send_event(str);
  }
}

void
Playfield::set_buttons(ButtonPanel* b)
{
  buttons = b;
  CaptureRectangle::set_action(makePinguAction(buttons->get_action_name())); 
}

void 
Playfield::set_pingu_info(PinguInfo* p)
{
  pingu_info = p;
}

void
Playfield::set_server(Server* s)
{
  server = s;
}

void
Playfield::set_client(Client* c)
{
  client = c;
}

/* EOF */

