// $Id: Editor.hh,v 1.17 1999/12/16 21:22:14 grumbel Exp $
// $Revision: 1.17 $
//
// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef EDITOR_HH
#define EDITOR_HH

#include <string>
#include <list>
#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "../PSMParser.hh"
#include "../PLF.hh"
#include "ObjectSelector.hh"
#include "StatusLine.hh"
#include "EditorObj.hh"

class Editor
{
private:
  PSMParser* psm;
  PLF*       plf;
  CL_Font*   font;
    
  //  list<surface_data> surfaces;

  //typedef list<surface_data>::iterator SurDataIter;

  list<EditorObj*> editor_objs;
  typedef list<EditorObj*>::iterator EditorObjIter;

  list<EditorObjIter> current_objs;
  typedef list<EditorObjIter>::iterator CurrentObjIter;

  string last_level;
  bool always_animate;
  int x_offset;
  int y_offset;
  int move_offset;
  string checkpoint;
  StatusLine status_line;
  ObjectSelector object_selector;

  // Other Level data
  string description;
  string levelname;
  vector<button_data> actions;
  background_data background;
  int    number_to_save;
  int    number_of_pingus;
  int    level_time;
  int    height;
  int    width;
  int    start_x_pos;
  int    start_y_pos;
  struct { float r;float g;float b; } bg;
public:
  Editor ();
  ~Editor ();

  string read_string (string prefix = "", string default_str = "");
  bool key_pressed(int key_no);
  void edit (string);
  EditorObjIter get_current_obj();
  void draw ();
  void draw_noflip();
  void move_obj();
  bool object_selected(EditorObjIter c_obj);
  void rect_get_current_objs();
  void delete_selection();
  void unmark(EditorObjIter c_obj);
  string basename(string);
  bool mouse_moved();
  void scroll();

  bool raise_obj(EditorObjIter);
  bool lower_obj(EditorObjIter);

  void raise_current_objs();
  void lower_current_objs();

  void   save_level (string);
  string save_tmp_level ();
  void   save_plf (string);
  void   save_psm (string);

  void load_level (string);
  void interactive_load();
};

#endif

/* EOF */

